/****************************************************************************/
/*	Purpose:	Driver for LCD Display 16x4									*/
/*	Version:	1.0															*/
/*	Author:		Mike Schaub													*/
/*	Date:		18.09.2005													*/
/*	www:		www.microcontroller-starterkits.de							*/
/*	email:																	*/
/*	Software:	WinARM to compile											*/
/*	Hardware:	LPC2194 with ARMEvoBoard									*/
/****************************************************************************/

#include <LCD_4x16.h>



void itoa(unsigned int value, unsigned char * s)
{
	unsigned int i;

	for (i=4;i>0;i--)
	{
		s[i] = (value % 10) + 0x30;
		value /= 10;
	}
}


void wait(unsigned int zeit)
{
	while (zeit > 0) zeit--;
}


void clrscr(void)
{
	RS = IN;
	lcd_out(CLS);
	wait(2000);
	lcd_out(HOME);
	wait(2000);
	RS = DA;
}


void home(void)
{
	RS = IN;
	lcd_out(HOME);
	wait(2000);
	RS = DA;
}


void fourth_row(void)
{
	RS = IN;
	lcd_out(0x80 | 0x50);
	wait(2000);
	RS = DA;
}


void third_row(void)
{
	RS = IN;
	lcd_out(0x80 | 0x10);
	wait(2000);
	RS = DA;
}


void second_row(void)
{
	RS = IN;
	lcd_out(0x80 | 0x40);
	wait(2000);
	RS = DA;
}


void first_row(void)
{
	RS = IN;
	lcd_out(0x80 | 0x00);
	wait(20000);
	RS = DA;
}


void lcd_init(unsigned char line, unsigned char dots)
{
	wait(15000);

	IOSET1 |= (LCD_D4 | LCD_D5);
	IOSET1 |= LCD_EN;
	IOCLR1 |= LCD_EN;
	wait(100);

	IOSET1 |= LCD_EN;
	IOCLR1 |= LCD_EN;
	wait(100);

	IOSET1 |= LCD_EN;
	IOCLR1 |= LCD_EN;
	wait(100);

	IOCLR1 |= LCD_D4;
	IOSET1 |= LCD_EN;
	wait(100);
	IOCLR1 |= LCD_EN;
	wait(10000);

	RS = IN;

	lcd_out(FUNKTION_SET | _4BIT | line | dots);
	lcd_out(CONTROL | DISPLAY_OFF);
	lcd_out(CONTROL | DISPLAY_ON | CURSOR_OFF | NO_BLINK);
	lcd_out(CLS);
	wait(100);
	lcd_out(HOME);
	wait(100);
	lcd_out(ENTRY | INCREMENT);
	RS = DA;
}


void lcd_out_data(unsigned char lcd_data)
{
	IOCLR1 |= (LCD_DATA);
	IOSET1 |= (((unsigned long) (lcd_data))<<20);
}


void lcd_out(char c)
{
	lcd_out_data((c & 0xF0) >> 4);
	if (RS) IOSET1 |= LCD_RS;
	else IOCLR1 |= LCD_RS;
	wait(2000);
	IOSET1 |= LCD_EN;
	wait(2000);
	IOCLR1 |= LCD_EN;
	wait(2000);

	lcd_out_data(c & 0x0F);
	if (RS) IOSET1 |= LCD_RS;
	else IOCLR1 |= LCD_RS;
	wait(2000);
	IOSET1 |= LCD_EN;
	wait(2000);
	IOCLR1 |= LCD_EN;
	wait(2000);
}


void put_lcd(const unsigned char * s)
{
	unsigned short int	i;

	i = 0;

	do
		{
			if (s[i] == '\0') break;
			if (i == 16) second_row();				
			if (i == 32) third_row();				
			if (i == 48) fourth_row();				
			if (i == 64)
				{
					first_row();
					i = 0;
				}
			switch (s[i])
			{
				case 0xE4:	lcd_out(0xE1);		//	''
							break;

				case 0xF6:	lcd_out(0xEF);		//	''
							break;

				case 0xFC:	lcd_out(0xF5);		//	''
							break;

				default:	lcd_out(s[i]);
							break;
			}

			i++;
			wait(2000);
		}

	while (s[i] != '\0');
}


void put_lcd_xy(unsigned int x, unsigned int y, const char * s)
{
	unsigned short int	i,z,help;

	i = 0;

	if (y < 4) z = y;
	else z = 0;

	switch (z)
		{
			case 0:	help = x;
					break;

			case 1:	help = x + 0x40;
					break;

			case 2:	help = x + 0x10;
					break;

			case 3:	help = x + 0x50;
					break;

			default:
					break;
		}

	RS = IN;
	lcd_out(0x80 | help);
	wait(2000);
	RS = DA;

	do
		{
			if (s[i] == '\0') break;
			if ((x == 16) && (y == 0))
				{
					second_row();
					x = 0;
					y = 1;
				}
			if ((x == 16) && (y == 1))
				{
					third_row();
					x = 0;
					y = 2;
				}
			if ((x == 16) && (y == 2))
				{
					fourth_row();
					x = 0;
					y = 3;
				}
			if ((x == 16) && (y == 3))
				{
					first_row();
					x = 0;
					y = 0;
				}

			switch (s[i])
			{
				case 0xE4:	lcd_out(0xE1);		//	''
							break;

				case 0xF6:	lcd_out(0xEF);		//	''
							break;

				case 0xFC:	lcd_out(0xF5);		//	''
							break;

				default:	lcd_out(s[i]);
							break;
			}

			i++;
			x++;
			wait(2000);
		}

	while (s[i] != '\0');
}

